@@ -1,12 +1,12 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
-from flask import redirect, url_for, render_template, make_response, request,\
-        jsonify
+from flask import redirect, url_for, render_template, make_response, request
 from flask.ext.login import login_required, current_user
 from .. import db
 from ..models import Request, ActionType
 from ..auth import PermissionType
 from ..auth.models import Permission, Division
+from ..util import jsonify
 
 
 @login_required
